#include "SymbolTable.cc"

/**
* SSymbol table is symbol table which supports scope.
*
* @author       Taeho Oh
* @version      0.0.1
* @since        1999/06/02
*/
class SSymbolTable
{
private:
	/**
	* The maximum scope.
	*/
	const int maxScope = 256 ;

	/**
	* The current scope.
	*/
	int scope ;

	/**
	* The symbol table list.
	*/
	SymbolTable *symbolTables[maxScope] ;

public:
	/**
	* Initialize SSymbolTable.
	*/
	SSymbolTable()
	{
		scope = 0 ;
		symbolTables[scope] = new SymbolTable() ;
	}

	/**
	* Delete all allocated ssymboltables.
	*/
	~SSymbolTable()
	{
		for(int i = 0; i <= scope; i++)
			delete symbolTables[i] ;
	}

	/**
	* Increment scope.
	*
	* @return none.
	*/
	void operator++()
	{
		if(scope == maxScope - 1)
		{
			fprintf(stderr, "Too many scope\n") ;
			return ;
		}
		else
			scope++ ;
		symbolTables[scope] = new SymbolTable() ;
	}

	/**
	* Increment scope.
	*
	* @return none.
	*/
	void operator++(int i)
	{
		operator++() ;
	}

	/**
	* Decrement scope.
	*
	* @return none.
	*/
	void operator--()
	{
		if(scope == 0)
		{
			fprintf(stderr, "No scope\n") ;
			return ;
		}
		else
		{
			delete symbolTables[scope] ;
			scope-- ;
		}
	}

	/**
	* Decrement scope.
	*
	* @return none.
	*/
	void operator--(int i)
	{
		operator--() ;
	}

	/**
	* Insert the symbol table entry to the current scope symbol table.
	*
	* @param lexeme lexeme which will be stored.
	* @param token the token value of lexeme.
	* @return none.
	*/
	void insert(const char * const lexeme, const int token, const int offset)
	{
		int global ;
		if(scope == 0)
			global = 1 ;
		else
			global = 0 ;
		symbolTables[scope]->insert(lexeme, token, offset, global) ;
	}

	/**
	* Look up the symbol table entry from the current scope symbol table.
	*
	* @param lexeme lexeme which will be used to find symbol table entry.
	* @return the token of lexeme.
	*/
	STEntry *lookup(const char * const lexeme) const
	{
		STEntry *entry = NULL ;
		int searchScope = scope ;
		
		entry = symbolTables[scope]->lookup(lexeme) ;

		while(searchScope != -1 && entry == NULL)
		{
			entry = symbolTables[searchScope]->lookup(lexeme) ;
			searchScope-- ;
		}

		return entry ;
	}

	/**
	* Get the current scope.
	*
	* @return current scope.
	*/
	int getScope()
	{
		return scope ;
	}
} ;
